"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Convert_instances, _Convert_getFirstRow, _Convert_forEachRow, _Convert_toCategory, _Convert_extractPointStyle, _Convert_extractValueStyle;
Object.defineProperty(exports, "__esModule", { value: true });
class Convert {
    constructor(modelizer) {
        _Convert_instances.add(this);
        this.getSeries = () => {
            return __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_getFirstRow).call(this)
                .cells()
                .map((col) => {
                const column = this.modelizer.getColumn(col.columnKey);
                const seriesStyle = column.style.get();
                return {
                    label: col.columnKey,
                    style: seriesStyle,
                };
            });
        };
        this.modelizer = modelizer;
    }
    toSeriesCategories() {
        if (__classPrivateFieldGet(this, _Convert_instances, "m", _Convert_getFirstRow).call(this)) {
            const series = this.getSeries();
            const categories = [];
            __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row) => {
                categories.push(__classPrivateFieldGet(this, _Convert_instances, "m", _Convert_toCategory).call(this, row));
            });
            return {
                series: series,
                categories: categories,
            };
        }
        return {
            series: [],
            categories: [],
        };
    }
    toVerticalLines(params) {
        const series = this.getSeries();
        const categories = [];
        const yValueOffset = params?.yValueOffset || 0;
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            categories.push({
                label: row.key,
                y: r + yValueOffset,
                values: row.cells().map((column) => column.toNumber()),
            });
        });
        return {
            series: series,
            categories: categories,
        };
    }
    toCombo() {
        const series = this.getSeries();
        const categories = [];
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            categories.push({
                label: row.key,
                y: r + 0.5,
                values: row.cells().map((column) => column.toNumber()),
                styles: __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractValueStyle).call(this, row.cells()),
            });
        });
        return {
            series: series,
            categories: categories,
        };
    }
    toScatter() {
        const series = this.getSeries();
        const categories = [];
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            categories.push({
                label: row.key,
                values: row.cells().map((cell) => {
                    return {
                        x: cell.getPoint(0)?.value,
                        y: cell.getPoint(1)?.value,
                    };
                }),
                styles: __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, row),
            });
        });
        return {
            series: series,
            categories: categories,
        };
    }
    toBubbles() {
        const series = this.getSeries();
        const categories = [];
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            categories.push({
                label: row.key,
                values: row.cells().map((col) => {
                    return {
                        size: Number(col.getPoint(0)?.value),
                        x: Number(col.getPoint(1)?.value),
                        y: Number(col.getPoint(2)?.value),
                    };
                }),
                styles: __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, row),
            });
        });
        return {
            series: series,
            categories: categories,
        };
    }
    toRowLabels() {
        const body = [];
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            body.push({
                values: [row.key],
                styles: __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, row),
            });
        });
        return {
            body,
        };
    }
    toColumnLabels() {
        let series = [];
        let styles = [];
        const firstRow = __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_getFirstRow).call(this);
        if (firstRow) {
            series = firstRow.cells().map((col) => col.columnKey);
            styles = __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, firstRow);
        }
        return {
            body: [
                {
                    values: series,
                    styles: styles,
                },
            ],
        };
    }
    toTableBody() {
        const body = [];
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            body.push({
                values: row.cells().map((cell) => cell.toCell()),
            });
        });
        return {
            body,
        };
    }
    toTable(params) {
        const body = [];
        const firstRow = __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_getFirstRow).call(this);
        if (!firstRow) {
            return {
                body: [
                    {
                        values: [],
                    },
                ],
            };
        }
        if (params?.showColumnLabels) {
            const header = [];
            if (params?.showRowLabels) {
                header.push("");
            }
            header.push(...firstRow.cells().map((col) => col.columnKey));
            body.push({
                values: header,
                // TODO: insert column label styles
            });
        }
        __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_forEachRow).call(this, (row, r) => {
            const tableRow = [];
            const tableRowStyles = [];
            if (params?.showRowLabels) {
                tableRow.push(String(row.key));
                tableRowStyles.push(row.style.get());
            }
            row.cells().forEach((cell, c) => {
                tableRow.push(cell.toCell());
            });
            tableRowStyles.push(...__classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, row));
            body.push({
                values: tableRow,
                styles: tableRowStyles,
            });
        });
        return {
            body,
        };
    }
    toResultRows() {
        const body = [];
        this.modelizer.processRows((row) => {
            const bodyRow = {
                key: row.key,
                cols: [],
            };
            row.cells().forEach((cell) => {
                const resultCol = {
                    key: cell.columnKey,
                    value: cell.getPoints(),
                };
                bodyRow.cols.push(resultCol);
            });
            body.push(bodyRow);
        });
        return body;
    }
}
_Convert_instances = new WeakSet(), _Convert_getFirstRow = function _Convert_getFirstRow() {
    return this.modelizer.getRow(0);
}, _Convert_forEachRow = function _Convert_forEachRow(cb) {
    this.modelizer.processRows(cb);
}, _Convert_toCategory = function _Convert_toCategory(row) {
    return {
        label: row.key,
        values: row.cells().map((column) => column.toNumberOrEmpty()),
        styles: __classPrivateFieldGet(this, _Convert_instances, "m", _Convert_extractPointStyle).call(this, row),
    };
}, _Convert_extractPointStyle = function _Convert_extractPointStyle(row) {
    const styles = [];
    row.cells().forEach((cell) => {
        const firstPoint = cell.getPoint();
        if (firstPoint?.style) {
            styles.push(firstPoint.style);
        }
        else {
            styles.push(null);
        }
    });
    return styles;
}, _Convert_extractValueStyle = function _Convert_extractValueStyle(cells) {
    return cells.map((cell) => cell.getPoint()?.style || null);
};
exports.default = Convert;
//# sourceMappingURL=convert.js.map