"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ResultInfo {
    constructor(result) {
        this.getPoint = (rowId, colId, valueId) => {
            const row = this.getRow(rowId);
            const column = this.getColumn(row, colId);
            return this.getValue(column, valueId);
        };
        this.getRow = (rowId) => {
            return this.result.body[rowId];
        };
        this.getColumn = (row, colId) => {
            if (row && row.cols) {
                return row.cols[colId];
            }
        };
        this.getValue = (column, valueId) => {
            if (!column || !column.value) {
                return;
            }
            else if (Array.isArray(column.value)) {
                valueId = valueId || 0;
                return column.value[valueId];
            }
            else {
                return column.value;
            }
        };
        this.getMeta = (point, key) => {
            if (point && point.meta) {
                const matchMeta = point.meta.filter((meta) => meta.key === key);
                if (matchMeta.length === 1) {
                    return matchMeta[0].value;
                }
                return matchMeta;
            }
        };
        this.hasMeta = (point, key) => {
            const hasMeta = this.getMeta(point, key);
            return (typeof hasMeta === "string" ||
                hasMeta === true ||
                (Array.isArray(hasMeta) && hasMeta.length) > 0);
        };
        this.getTag = (point, categoryId) => {
            if (point && point.tags) {
                return point.tags.find((tag) => tag.categoryId === categoryId);
            }
        };
        this.result = result;
    }
    rowCount() {
        return this.result.body.length;
    }
    rowKeys() {
        return this.result.body.map((row) => row.key);
    }
    getRowKeys() {
        return this.result.body.map((row) => row.key);
    }
    getColumnKeys() {
        if (!this.result.body[0])
            return [];
        return this.result.body[0].cols.map((col) => col.key);
    }
}
exports.default = ResultInfo;
//# sourceMappingURL=resultInfo.js.map