"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Modelizer_instances, _Modelizer_keys, _Modelizer_rows, _Modelizer_columns, _Modelizer_cells, _Modelizer_meta, _Modelizer_addKey, _Modelizer_getKeys, _Modelizer_addKeyByMode, _Modelizer_setKeys, _Modelizer_updateKey, _Modelizer_getKey, _Modelizer_style, _Modelizer_modelMeta, _Modelizer_findCellByIndex, _Modelizer_findCellByKeys, _Modelizer_pushCells, _Modelizer_filterCells, _Modelizer_parseCellKeys, _Modelizer_parseCellKey, _Modelizer_createOrFailKey, _Modelizer_defaultCell, _Modelizer_filterCellKeys, _Modelizer_handleError;
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importStar(require("lodash"));
const points_1 = __importDefault(require("../points"));
const dump_1 = require("./dump");
/**
 * Modelizer class needs some datapoints to work. Each datapoint will add
 * a key to #keys.row and #keys.col. All #keys will define the rows and
 * columns of the output table. Rows and columns can be added, removed, sorted
 * and modelized.
 */
class Modelizer {
    constructor(options, query) {
        _Modelizer_instances.add(this);
        /**
         * Stores all keys for the current set of rows and columns.
         * Represents the current edges of result table.
         * @private
         */
        _Modelizer_keys.set(this, {
            row: [],
            column: [],
        });
        /**
         * Stores all Model rows. Each model holds an array of referenced cells and
         * some useful methods to modelize row data.
         * Referenced cells of a row represent its columns.
         * @private
         */
        _Modelizer_rows.set(this, []);
        /**
         * Stores all Model columns. Each model holds an array of referenced cells and
         * some useful methods to modelize column data.
         * Referenced cells of a column represent its rows.
         * @private
         */
        _Modelizer_columns.set(this, []);
        /**
         * Stores all Cells. Cells are referenced by rows and columns.
         * @private
         */
        _Modelizer_cells.set(this, []);
        /**
         * Stores an array of Metadata for the current instance. Useful to buffer e.g.
         * Min/Max information.
         * @private
         */
        _Modelizer_meta.set(this, []);
        _Modelizer_style.set(this, () => {
            const style = {
                state: {},
                assign(assignStyle) {
                    Object.assign(style.state, assignStyle);
                },
                get() {
                    return style.state;
                },
            };
            return style;
        });
        _Modelizer_modelMeta.set(this, () => {
            const meta = {
                state: [],
                set(key, value) {
                    meta.state.push({
                        key: key,
                        value: value,
                    });
                },
                get(key) {
                    return meta.state.find((state) => state.key === key);
                },
            };
            return meta;
        });
        this.strict = options?.strict !== undefined ? options?.strict : true;
        if (options?.points) {
            this.addPoints(options.points);
        }
        this.query = query;
    }
    /**
     * Add an array of Datapoints to Modelizer. Each datapoint will create a
     * corresponding row- or column key, if not existing already.
     * Each point will create a cell.
     * If a point is added to an existing cell (because row and col key equal an
     * existing keypair), the point will push existing points in the target cell.
     * @param points An array of datapoints
     * @param options Specify where the target row-/column-keys are coming from
     * @return {this}
     */
    addPoints(points, options) {
        points.forEach((point) => {
            this.addPoint(point, options);
        });
        return this;
    }
    /**
     * Add a single Datapoint to the table. If the point matches an already
     * existing Cell by row and column key, the point will be added to the
     * stack of points of the matching Cell. Otherwise, a new row- and/or
     * column-key will be created according to the given datapoint.
     * @param point A datapoint
     * @param options Specify where the target row-/column-keys are coming from
     * @returns {this}
     */
    addPoint(point, options) {
        const rowKey = options?.rowKey ? options.rowKey(point) : point.row;
        const colKey = options?.colKey ? options.colKey(point) : point.column;
        const r = this.addRow(rowKey);
        const c = this.addColumn(colKey);
        const cell = this.pushCellPoints(r, c, point);
        if (options?.value) {
            const value = options.value(point);
            cell.setValue(value);
        }
        else if (cell.value === undefined) {
            cell.setValue(point.value);
        }
        return this;
    }
    /**
     * Apply a callback to each cell of the current table.
     * @returns {this}
     * @param cb The callback to run on each cell
     */
    process(cb) {
        this.getKeys("row").forEach((rowKey, r) => {
            this.getKeys("column").forEach((colKey, c) => {
                const cell = this.getCell(r, c);
                cb(cell, r, c, rowKey, colKey);
            });
        });
        return this;
    }
    /**
     * Pass a callback to run on each row or column by given mode.
     * @param mode KeyMode is either 'row' or 'col'
     * @param cb The callback to run on each row
     */
    processByMode(mode, cb) {
        switch (mode) {
            case "row":
                return this.processRows(cb);
            case "column":
                return this.processColumns(cb);
        }
        return this;
    }
    /**
     * Pass a callback to run on each row.
     * @param cb The callback to run on each row
     */
    processRows(cb) {
        this.getKeys("row").forEach((rowKey, r) => {
            const row = this.getRow(r);
            cb(row, r, rowKey);
        });
        return this;
    }
    /**
     * Pass a callback to run on each column.
     * @param cb The callback to run on each column
     */
    processColumns(cb) {
        this.getKeys("column").forEach((colKey, c) => {
            const column = this.getColumn(c);
            cb(column, c, colKey);
        });
        return this;
    }
    /**
     * Retreive the current keys for row or column.
     * @param mode Pass "row" or "col" to specify the target dimension.
     * @returns {string[]} Array of strings containing the keys.
     */
    getKeys(mode) {
        const tmpKeys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode);
        if (Array.isArray(tmpKeys)) {
            return [...__classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode)];
        }
        return [];
    }
    /**
     * Pass a string key to append a new row with that name.
     * @param key
     * @returns {number} Column index of the created row.
     */
    addRow(key) {
        return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_addKey).call(this, key, "row");
    }
    /**
     * Pass a string key to append a new column with that name.
     * @param key
     * @returns {number} Column index of the created column.
     */
    addColumn(key) {
        return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_addKey).call(this, key, "column");
    }
    setCellValueByMode(mode, k1, k2, value) {
        if (mode === "row") {
            return this.setCellValue(k1, k2, value);
        }
        else {
            return this.setCellValue(k2, k1, value);
        }
    }
    /**
     * Pass a {CellValue} to a target {Cell}.
     * @param r Pass a number or a string to determine the target cell row.
     * @param c Pass a row or column key to target the cell.
     * @param value A string or a number to alter the cell value.
     * @return {Cell}
     */
    setCellValue(r, c, value) {
        const targetCell = this.getCell(r, c);
        targetCell.setValue(value);
        return targetCell;
    }
    /**
     * Pass a Datapoint to a target Cell. The point will not change the
     * target cell value.
     * @param r Pass a number or a string to determine the target cell row.
     * @param c Pass a row or column key to target the cell.
     * @param point The point to push the cell's point stack with.
     */
    pushCellPoints(r, c, point) {
        const targetCell = this.getCell(r, c);
        targetCell.points.push(point);
        return targetCell;
    }
    /**
     * Retreive a rows- or columns-model object for the given selector.
     * @param mode Pass 'row' or 'column'.
     * @param key Pass a number or a string to select the target row/column.
     * @return A model row/column object containing all cells.
     */
    getByMode(mode, key) {
        if (mode === "row") {
            return this.getRow(key);
        }
        else {
            return this.getColumn(key);
        }
    }
    getRow(key) {
        return this.findOrCreateModel(__classPrivateFieldGet(this, _Modelizer_rows, "f"), "row", key);
    }
    getColumn(key) {
        return this.findOrCreateModel(__classPrivateFieldGet(this, _Modelizer_columns, "f"), "column", key);
    }
    findOrCreateModel(models, mode, key) {
        const modelKey = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKey).call(this, key, mode);
        const existing = models.find((model) => model.key === modelKey);
        if (existing) {
            return existing;
        }
        const model = this.createModel(modelKey, mode);
        models.push(model);
        return model;
    }
    /**
     * Retreive a row or column model object for the given selector.
     * @param key Pass a number or a string to set the target key.
     * @param mode Specify "row" or "column"
     * @return A model object containing all cells.
     */
    createModel(key, mode) {
        const model = {
            key: key,
            mode: mode,
            id: () => __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, key, model.mode),
            style: __classPrivateFieldGet(this, _Modelizer_style, "f").call(this),
            cells: () => __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_filterCells).call(this, model.mode === "row" ? "column" : "row", key),
            meta: __classPrivateFieldGet(this, _Modelizer_modelMeta, "f").call(this),
            each: (cb) => {
                model.cells().forEach((cell) => cb(cell));
                return model;
            },
            collect: (cb) => {
                const values = [];
                model.each((cell) => {
                    if (!cb || (typeof cb === "function" && cb(cell))) {
                        values.push(cell.toNumber());
                    }
                });
                return values;
            },
            getCell: (i) => this.getCellByMode(model.mode, model.id(), i),
            setCell: (i, cell) => {
                this.setCellByMode(model.mode, model.id(), i, cell);
                return model;
            },
            setCellValue: (i, value) => {
                this.setCellValueByMode(model.mode, model.id(), i, value);
                return model;
            },
            updateKey: (newKey) => {
                const targetKey = model.mode === "row" ? "rowKey" : "columnKey";
                model.cells().forEach((cell) => {
                    cell.points.forEach((point) => {
                        point[model.mode] = newKey;
                    });
                    cell[targetKey] = newKey;
                });
                __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_updateKey).call(this, model.mode, model.id(), newKey);
                return model;
            },
            dump: (s1, s2) => model.mode === "row"
                ? this.dump(s1, s2, [model.id()], [])
                : this.dump(s1, s2, [], [model.id()]),
        };
        return model;
    }
    getCellByMode(mode, i1, i2) {
        if (mode === "row") {
            return this.getCell(i1, i2);
        }
        else {
            return this.getCell(i2, i1);
        }
    }
    /**
     * Retrieve a cell by row and column key.
     * If no cell matches the given keys, a new cell will be created (non-strict).
     * In strict mode, Modelizer will throw an error.
     * @param r Set a numeric or string value to address the target row
     * @param c Set a numeric or string value to address the target column
     * @returns {Cell}
     */
    getCell(r, c) {
        const rowId = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, r, "row");
        const colId = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, c, "column");
        return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByIndex).call(this, rowId, colId);
    }
    setCellByMode(mode, k1, k2, cell) {
        if (mode === "row") {
            return this.setCell(k1, k2, cell);
        }
        else {
            return this.setCell(k2, k1, cell);
        }
    }
    /**
     * Update a Cell object at the given keys or create one.
     * In strict mode, Modelizer will throw an error when there are no matching keys.
     * @param rowKey Set a numeric or string value to address the target row
     * @param colKey Set a numeric or string value to address the target column
     * @param cell Optionally set a numeric or string value to address the target column
     * @returns {Cell}
     */
    setCell(rowKey, colKey, cell) {
        const r = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, rowKey, "row");
        const c = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, colKey, "column");
        if (cell) {
            const targetCell = lodash_1.default.cloneDeep(cell);
            if (typeof rowKey === "string") {
                targetCell.rowKey = rowKey;
            }
            if (typeof colKey === "string") {
                targetCell.columnKey = colKey;
            }
            const createdCell = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByKeys).call(this, targetCell.rowKey, targetCell.columnKey);
            Object.assign(createdCell, targetCell);
            return createdCell;
        }
        return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByIndex).call(this, r, c);
    }
    getCells() {
        return __classPrivateFieldGet(this, _Modelizer_cells, "f");
    }
    setMeta(key, value) {
        __classPrivateFieldGet(this, _Modelizer_meta, "f").push({
            key,
            value,
        });
    }
    getMeta(key) {
        return __classPrivateFieldGet(this, _Modelizer_meta, "f").find((meta) => meta.key === key);
    }
    getNestedMeta(key, label) {
        const nestedMeta = this.getMeta(key)?.value;
        if (Array.isArray(nestedMeta)) {
            return nestedMeta.find((meta) => meta.label === label)?.value;
        }
    }
    getMetas() {
        return __classPrivateFieldGet(this, _Modelizer_meta, "f");
    }
    /**
     * Get the first of all DataPoints. This is helpful if you need general tags
     * or meta information and if you can assure that all active datapoints
     * hold the same information as Point zero.
     * @returns {DataPoint}
     */
    getFirstPoint() {
        let firstPoint;
        this.process((cell) => {
            if (firstPoint) {
                return;
            }
            const cellHasPoint = cell.getPoint(0, false);
            if (cellHasPoint) {
                firstPoint = cellHasPoint;
            }
        });
        return firstPoint;
    }
    /**
     * Specify "row" or "column" and an array of keys to filter and sort the target
     * dimension of current table.
     * @param mode
     * @param keys
     */
    sort(mode, keys) {
        const ids = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKeys).call(this, keys, mode);
        const existingKeys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode);
        const sortedKeys = [];
        ids.forEach((id) => {
            sortedKeys.push(existingKeys[id]);
        });
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_setKeys).call(this, mode, sortedKeys);
        return this;
    }
    /**
     * Transpose the result by switching rows and columns.
     */
    transpose() {
        const sortCols = this.getKeys("column");
        const sortRows = this.getKeys("row");
        const tmpRows = ([] = []);
        sortRows.forEach((rowKey) => {
            tmpRows.push(this.getRow(rowKey));
        });
        const tmpColumns = [];
        sortCols.forEach((columnKey) => {
            tmpColumns.push(this.getColumn(columnKey));
        });
        __classPrivateFieldGet(this, _Modelizer_keys, "f").row = [];
        __classPrivateFieldGet(this, _Modelizer_keys, "f").column = [];
        __classPrivateFieldSet(this, _Modelizer_rows, [], "f");
        __classPrivateFieldSet(this, _Modelizer_columns, [], "f");
        tmpColumns.forEach((column) => {
            column.mode = "row";
            __classPrivateFieldGet(this, _Modelizer_rows, "f").push(column);
            __classPrivateFieldGet(this, _Modelizer_keys, "f").row.push(column.key);
        });
        tmpRows.forEach((row) => {
            row.mode = "column";
            __classPrivateFieldGet(this, _Modelizer_columns, "f").push(row);
            __classPrivateFieldGet(this, _Modelizer_keys, "f").column.push(row.key);
        });
        __classPrivateFieldGet(this, _Modelizer_cells, "f").forEach((cell) => {
            const rowKey = cell.rowKey;
            const colKey = cell.columnKey;
            cell.points.forEach((point) => {
                point.row = colKey;
                point.column = rowKey;
            });
            cell.rowKey = colKey;
            cell.columnKey = rowKey;
        });
    }
    /**
     * Write the contents of current table to the console.
     * @param firstColSize Specify the number of characters for label width.
     * @param colSize Number of characters for body column width.
     * @param rows Restrict the outputted rows to a given set of keys.
     * @param cols Restrict the outputted columns to a given set of keys.
     * @param renderCell Pass a callback to display customized information.
     */
    dump(firstColSize, colSize, rows, cols, renderCell) {
        firstColSize = firstColSize || 15;
        colSize = colSize || 10;
        const rowKeys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_filterCellKeys).call(this, rows, "row");
        const colKeys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_filterCellKeys).call(this, cols, "column");
        (0, dump_1.dumpHeader)(firstColSize, colSize, colKeys);
        (0, dump_1.dumpBody)(firstColSize, colSize, rowKeys, colKeys, __classPrivateFieldGet(this, _Modelizer_cells, "f"), renderCell);
        (0, dump_1.dumpFooter)(firstColSize, colSize, colKeys);
    }
}
_Modelizer_keys = new WeakMap(), _Modelizer_rows = new WeakMap(), _Modelizer_columns = new WeakMap(), _Modelizer_cells = new WeakMap(), _Modelizer_meta = new WeakMap(), _Modelizer_style = new WeakMap(), _Modelizer_modelMeta = new WeakMap(), _Modelizer_instances = new WeakSet(), _Modelizer_addKey = function _Modelizer_addKey(key, mode) {
    const keys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode);
    if (!keys.includes(key)) {
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_addKeyByMode).call(this, mode, key);
    }
    return keys.indexOf(key);
}, _Modelizer_getKeys = function _Modelizer_getKeys(mode) {
    return __classPrivateFieldGet(this, _Modelizer_keys, "f")[mode] || [];
}, _Modelizer_addKeyByMode = function _Modelizer_addKeyByMode(mode, key) {
    __classPrivateFieldGet(this, _Modelizer_keys, "f")[mode].push(key);
}, _Modelizer_setKeys = function _Modelizer_setKeys(mode, keys) {
    __classPrivateFieldGet(this, _Modelizer_keys, "f")[mode] = keys;
}, _Modelizer_updateKey = function _Modelizer_updateKey(mode, index, key) {
    if (!__classPrivateFieldGet(this, _Modelizer_keys, "f")[mode][index]) {
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_handleError).call(this, "Can't update key at index " + index);
    }
    __classPrivateFieldGet(this, _Modelizer_keys, "f")[mode][index] = key;
}, _Modelizer_getKey = function _Modelizer_getKey(key, mode) {
    const id = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, key, mode);
    return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode)[id];
}, _Modelizer_findCellByIndex = function _Modelizer_findCellByIndex(r, c) {
    const rowKey = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKey).call(this, r, "row");
    const columnKey = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKey).call(this, c, "column");
    if (!rowKey || !columnKey) {
        throw "Valid keys are required to find cell at r: " + r + ", c: " + c;
    }
    return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByKeys).call(this, rowKey, columnKey);
}, _Modelizer_findCellByKeys = function _Modelizer_findCellByKeys(rowKey, columnKey) {
    const existingCell = __classPrivateFieldGet(this, _Modelizer_cells, "f").find((cell) => cell.rowKey === rowKey && cell.columnKey === columnKey);
    if (existingCell) {
        return existingCell;
    }
    const createdCell = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_defaultCell).call(this, rowKey, columnKey);
    __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_pushCells).call(this, createdCell);
    return createdCell;
}, _Modelizer_pushCells = function _Modelizer_pushCells(cell) {
    __classPrivateFieldGet(this, _Modelizer_cells, "f").push(cell);
}, _Modelizer_filterCells = function _Modelizer_filterCells(mode, key) {
    const keys = this.getKeys(mode);
    const cells = [];
    keys.forEach((targetKey) => {
        const findOrCreateCell = mode === "column"
            ? __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByKeys).call(this, key, targetKey)
            : __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_findCellByKeys).call(this, targetKey, key);
        cells.push(findOrCreateCell);
    });
    return cells;
}, _Modelizer_parseCellKeys = function _Modelizer_parseCellKeys(keys, mode) {
    if (!keys || keys.map === undefined)
        return [];
    return keys.map((key) => __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, key, mode));
}, _Modelizer_parseCellKey = function _Modelizer_parseCellKey(key, mode) {
    const keys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode);
    if (typeof key === "number") {
        if (keys[key])
            return key;
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_handleError).call(this, `Key of '${mode}' not found: ${key}. Length is: ${keys.length}`);
    }
    if (typeof key === "string") {
        const index = keys.indexOf(key);
        if (index < 0) {
            return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_createOrFailKey).call(this, key, mode, keys);
        }
        return index;
    }
    __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_handleError).call(this, `Could not determine '${mode}' key: ${key}`);
}, _Modelizer_createOrFailKey = function _Modelizer_createOrFailKey(key, mode, keys) {
    if (this.strict) {
        const available = keys.join(" | ");
        const message = `Key of '${mode}' not found: ${key}. Available keys are: ${available}`;
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_handleError).call(this, message);
    }
    else {
        __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_addKey).call(this, key, mode);
        return __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKey).call(this, key, mode);
    }
}, _Modelizer_defaultCell = function _Modelizer_defaultCell(rowKey, columnKey) {
    const cell = {
        value: undefined,
        rowKey: rowKey,
        columnKey: columnKey,
        meta: {},
        targetPoint: 0,
        setTargetPoint: (i) => {
            cell.targetPoint = i || 0;
            return cell;
        },
        getTargetPoint: () => {
            return cell.targetPoint;
        },
        points: [],
        getPoint: (i, forceCreate) => {
            const points = cell.getPoints();
            i = i === undefined ? cell.getTargetPoint() : i;
            i = points[i] ? i : 0;
            if (!points || !points[i]) {
                if (forceCreate === true) {
                    const addPoint = cell.createPoint();
                    cell.addPoint(addPoint);
                    return addPoint;
                }
                return null;
            }
            return points[i];
        },
        getPoints: () => {
            return cell.points || [];
        },
        setPoints: (points) => {
            cell.points = points;
            if (points.length) {
                cell.setValue(points[0].value);
            }
            else {
                cell.setValue(null);
            }
        },
        createPoint: (value) => {
            return points_1.default.dataPointFactory(rowKey, columnKey, [], [], value);
        },
        addPoint: (point) => {
            if (typeof point !== "object") {
                point = cell.createPoint(point);
            }
            cell.points.push(point);
            return cell;
        },
        getValue: () => cell.value || cell.getPoint()?.value,
        setValue: (value) => {
            cell.getPoint(0, true).value = value;
            cell.value = value;
            return cell;
        },
        getRow: () => this.getRow(cell.rowKey),
        getColumn: () => this.getColumn(cell.columnKey),
        toNumber: (precision) => {
            let currentValue = cell.getValue() || 0;
            if (typeof currentValue === "string" && currentValue.includes(",")) {
                currentValue = currentValue
                    .replace(",", ".")
                    .replace("±", "")
                    .replace("+", "");
            }
            currentValue = Number(currentValue);
            if (precision !== undefined) {
                return (0, lodash_1.round)(currentValue, precision);
            }
            return currentValue;
        },
        toNumberOrEmpty: () => {
            let currentValue = cell.toCell();
            if (currentValue === "") {
                return currentValue;
            }
            return cell.toNumber();
        },
        toCell: () => {
            let currentValue = cell.getValue();
            if (currentValue === false ||
                currentValue === null ||
                currentValue === undefined ||
                currentValue === "") {
                return "";
            }
            return currentValue;
        },
        dump: () => (0, dump_1.dumpCell)(cell),
    };
    return cell;
}, _Modelizer_filterCellKeys = function _Modelizer_filterCellKeys(keys, mode) {
    const ids = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_parseCellKeys).call(this, keys, mode);
    const allKeys = __classPrivateFieldGet(this, _Modelizer_instances, "m", _Modelizer_getKeys).call(this, mode);
    return allKeys.filter((key, c) => ids.length === 0 || ids.includes(c));
}, _Modelizer_handleError = function _Modelizer_handleError(message) {
    throw new Error(message);
};
exports.default = Modelizer;
//# sourceMappingURL=modelizer.js.map