"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Generic = void 0;
const node_xlsx_1 = __importDefault(require("node-xlsx"));
const parser_1 = require("./parser");
const fs = require("fs");
const path = require("path");
class Generic extends parser_1.Parser {
    constructor(config) {
        super(config);
    }
    async fromXlsx(file) {
        this.file = path.basename(file);
        const worksheetId = this.config.worksheetId || 0;
        const workSheetsFromBuffer = node_xlsx_1.default.parse(fs.readFileSync(file));
        const data = workSheetsFromBuffer[worksheetId].data;
        console.log("File rows count: " + String(data.length));
        this.autoDetectConfig(data);
        console.log("Table separator: " + this.tableSeparator);
        data.forEach((row) => {
            this.parseSections(row);
        });
        this.setDatasheets();
        return this.datasheets;
    }
    autoDetectConfig(data) {
        this.tableSeparator = !this.config.separator
            ? data[0][0]
            : this.config.separator;
    }
    parseSections(data) {
        const firstCell = String(data[0]).trim();
        const secondCell = data[1];
        const hasFirstCell = data[0] && String(data[0]).length > 0;
        const hasSecondCell = typeof secondCell !== "undefined" && String(data[1]).length > 0;
        if (firstCell.indexOf(this.tableSeparator) !== -1) {
            this.count++;
            this.currentSection = "info";
            this.results[this.count] = {
                info: [
                    {
                        key: "file",
                        value: this.file,
                    },
                ],
                header: [],
                body: [],
                meta: [],
            };
        }
        if ((firstCell.length === 0 && !hasSecondCell) ||
            this.config.skipRows.indexOf(firstCell) > -1) {
            return;
        }
        if (this.results[this.count]) {
            if (hasFirstCell && !hasSecondCell) {
                this.results[this.count].info.push({
                    key: this.currentSection,
                    value: firstCell,
                });
            }
            if (hasFirstCell && this.config.firstCell === firstCell) {
                this.currentSection = "header";
                this.results[this.count].header.push(data);
                return;
            }
            if (!hasFirstCell && hasSecondCell && this.currentSection !== "body") {
                this.currentSection = "header";
                this.results[this.count].header.push(data.slice(1));
            }
            if (hasFirstCell && hasSecondCell) {
                this.currentSection = "body";
                this.results[this.count].body.push(data);
            }
        }
    }
}
exports.Generic = Generic;
//# sourceMappingURL=generic.js.map