import { PrismaClient, Tag } from "./client";
import { CachedObject, CategoryCount, DataGrid, DataGridTransformation, DataPoint, DataPointModifier, Datasheet, DataTag, DataTagSelector, IdSelector, QueryOptions, QueryResult, QueryTransformationDump, QueryTransformationError, RawResult, Selector, Sheets } from "./types/types";
import Points from "./points";
import Modelizer from "./modelizer/modelizer";
import Keys from "./keys";
export default class Query {
    prisma: PrismaClient | any;
    grid: DataGrid;
    options: QueryOptions;
    modelizer: Modelizer;
    private selector;
    private dataTagSelector;
    private nonGreedySelector;
    private maxSheets;
    private cache;
    constructor(prisma: PrismaClient | any);
    static run(options: QueryOptions): Promise<QueryResult>;
    setOptions(options: QueryOptions): this;
    compute(): Promise<QueryResult>;
    getRawResult(allTagIds: any): Promise<RawResult>;
    fromCache(cached: CachedObject): CachedObject;
    getTagInfo(tagIds: number[]): Promise<Tag[]>;
    findSheets(tags: Tag[], isNonGreedy: boolean): Promise<Datasheet[]>;
    parseSheets(sheets: Sheets): Datasheet[];
    filterSheets(sheets: Sheets): Sheets;
    getSheetCategoryCount(sheets: Sheets): CategoryCount[];
    extractDataPoints(sheets: Datasheet[]): DataPoint[];
    modifyDataPoints(dataPoints: DataPoint[], level: number): Points;
    getDatapointModifiersByLevel(level: number): DataPointModifier[];
    parseSelector(selector: Selector): Promise<IdSelector[]>;
    convertDataTagSelector(selector: DataTagSelector): Promise<Selector>;
    getTagIds(tags: DataTag[]): Promise<number[]>;
    getTags(tags: DataTag[]): Promise<DataTag[]>;
    setCategoryId(tag: DataTag): Promise<void>;
    setTagId(tag: DataTag): Promise<void>;
    transformResult(transformations: DataGridTransformation[], modelizer: Modelizer, inputKeys: Keys, errors: QueryTransformationError[], dump: QueryTransformationDump): Promise<void>;
    checkCondition(transform: any, modelizer: Modelizer): Promise<boolean>;
}
