import { Category, Prisma, PrismaClient, Tag } from "./client";
import Query from "./query";
import { Datasheet, DataTag, StatusTracker, StoreOptions, StoreSummary } from "./types/types";
type CreateSheetData = Prisma.SheetCreateArgs["data"] & {
    tags: {
        connect: CreateSheetTagsConnect;
    };
};
type CreateSheetTagsConnect = Prisma.Enumerable<Prisma.TagWhereUniqueInput>;
export declare class Store {
    prisma: PrismaClient;
    summary: StoreSummary;
    options: StoreOptions | undefined;
    query: Query;
    categories: Category[];
    sheetKeys: Record<string, number>;
    tags: Tag[];
    importId: number;
    status: StatusTracker;
    tagValueMaxLength: number;
    constructor(prisma: PrismaClient, options?: StoreOptions);
    run(datasheets: Datasheet[]): Promise<StoreSummary>;
    runBefore(): Promise<void>;
    getImport(): Promise<void>;
    getCategories(): Promise<void>;
    createCategory(name: string): Promise<Category>;
    getTags(): Promise<void>;
    createTag(name: string, catId: number): Promise<Tag>;
    storeData(datasheet: Datasheet): Promise<void>;
    createSheetData(datasheet: Datasheet): CreateSheetData;
    deleteExistingDatasheets(createSheetData: CreateSheetData): Promise<void>;
    createSheet(datasheet: Datasheet, createSheetData: CreateSheetData): Promise<{
        id: number;
        importId: number;
        columns: string;
        rows: string;
        data: string;
        meta: string;
        rowKey: string;
        columnKey: string;
        tagKey: string;
        tagInfo: string;
    }>;
    dedupTagKeys(sheetData: CreateSheetData): Promise<void>;
    addCategoryIdToTags(tags: DataTag[]): Promise<void>;
    addTagIdsToTags(tags: DataTag[]): Promise<void>;
    getSheetKeys(datasheet: Datasheet): {
        rowKey: string;
        columnKey: string;
        tagKey: string;
        sheetKey: any;
    };
    getTagKey(tags: any): string;
    createHashFromObject(obj: Object): string;
    createHashFromArray(arr: string[]): string;
    createHash(str: string): string;
}
export {};
